@@ -1,111 +1,62 @@
 /*
  * Copyright (c) 1999-2000 Image Power, Inc. and the University of
  *   British Columbia.
- * Copyright (c) 2001-2002 Michael David Adams.
+ * Copyright (c) 2001-2003 Michael David Adams.
  * All rights reserved.
  */
 
 /* __START_OF_JASPER_LICENSE__
  * 
- * JasPer Software License
+ * JasPer License Version 2.0
  * 
- * IMAGE POWER JPEG-2000 PUBLIC LICENSE
- * ************************************
+ * Copyright (c) 2001-2006 Michael David Adams
+ * Copyright (c) 1999-2000 Image Power, Inc.
+ * Copyright (c) 1999-2000 The University of British Columbia
  * 
- * GRANT:
+ * All rights reserved.
  * 
- * Permission is hereby granted, free of charge, to any person (the "User")
- * obtaining a copy of this software and associated documentation, to deal
- * in the JasPer Software without restriction, including without limitation
- * the right to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the JasPer Software (in source and binary forms),
- * and to permit persons to whom the JasPer Software is furnished to do so,
- * provided further that the License Conditions below are met.
+ * Permission is hereby granted, free of charge, to any person (the
+ * "User") obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without restriction,
+ * including without limitation the rights to use, copy, modify, merge,
+ * publish, distribute, and/or sell copies of the Software, and to permit
+ * persons to whom the Software is furnished to do so, subject to the
+ * following conditions:
  * 
- * License Conditions
- * ******************
+ * 1.  The above copyright notices and this permission notice (which
+ * includes the disclaimer below) shall be included in all copies or
+ * substantial portions of the Software.
  * 
- * A.  Redistributions of source code must retain the above copyright notice,
- * and this list of conditions, and the following disclaimer.
+ * 2.  The name of a copyright holder shall not be used to endorse or
+ * promote products derived from the Software without specific prior
+ * written permission.
  * 
- * B.  Redistributions in binary form must reproduce the above copyright
- * notice, and this list of conditions, and the following disclaimer in
- * the documentation and/or other materials provided with the distribution.
- * 
- * C.  Neither the name of Image Power, Inc. nor any other contributor
- * (including, but not limited to, the University of British Columbia and
- * Michael David Adams) may be used to endorse or promote products derived
- * from this software without specific prior written permission.
- * 
- * D.  User agrees that it shall not commence any action against Image Power,
- * Inc., the University of British Columbia, Michael David Adams, or any
- * other contributors (collectively "Licensors") for infringement of any
- * intellectual property rights ("IPR") held by the User in respect of any
- * technology that User owns or has a right to license or sublicense and
- * which is an element required in order to claim compliance with ISO/IEC
- * 15444-1 (i.e., JPEG-2000 Part 1).  "IPR" means all intellectual property
- * rights worldwide arising under statutory or common law, and whether
- * or not perfected, including, without limitation, all (i) patents and
- * patent applications owned or licensable by User; (ii) rights associated
- * with works of authorship including copyrights, copyright applications,
- * copyright registrations, mask work rights, mask work applications,
- * mask work registrations; (iii) rights relating to the protection of
- * trade secrets and confidential information; (iv) any right analogous
- * to those set forth in subsections (i), (ii), or (iii) and any other
- * proprietary rights relating to intangible property (other than trademark,
- * trade dress, or service mark rights); and (v) divisions, continuations,
- * renewals, reissues and extensions of the foregoing (as and to the extent
- * applicable) now existing, hereafter filed, issued or acquired.
- * 
- * E.  If User commences an infringement action against any Licensor(s) then
- * such Licensor(s) shall have the right to terminate User's license and
- * all sublicenses that have been granted hereunder by User to other parties.
- * 
- * F.  This software is for use only in hardware or software products that
- * are compliant with ISO/IEC 15444-1 (i.e., JPEG-2000 Part 1).  No license
- * or right to this Software is granted for products that do not comply
- * with ISO/IEC 15444-1.  The JPEG-2000 Part 1 standard can be purchased
- * from the ISO.
- * 
- * THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
- * NO USE OF THE JASPER SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER
- * THIS DISCLAIMER.  THE JASPER SOFTWARE IS PROVIDED BY THE LICENSORS AND
- * CONTRIBUTORS UNDER THIS LICENSE ON AN ``AS-IS'' BASIS, WITHOUT WARRANTY
- * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT LIMITATION,
- * WARRANTIES THAT THE JASPER SOFTWARE IS FREE OF DEFECTS, IS MERCHANTABLE,
- * IS FIT FOR A PARTICULAR PURPOSE OR IS NON-INFRINGING.  THOSE INTENDING
- * TO USE THE JASPER SOFTWARE OR MODIFICATIONS THEREOF FOR USE IN HARDWARE
- * OR SOFTWARE PRODUCTS ARE ADVISED THAT THEIR USE MAY INFRINGE EXISTING
- * PATENTS, COPYRIGHTS, TRADEMARKS, OR OTHER INTELLECTUAL PROPERTY RIGHTS.
- * THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE JASPER SOFTWARE
- * IS WITH THE USER.  SHOULD ANY PART OF THE JASPER SOFTWARE PROVE DEFECTIVE
- * IN ANY RESPECT, THE USER (AND NOT THE INITIAL DEVELOPERS, THE UNIVERSITY
- * OF BRITISH COLUMBIA, IMAGE POWER, INC., MICHAEL DAVID ADAMS, OR ANY
- * OTHER CONTRIBUTOR) SHALL ASSUME THE COST OF ANY NECESSARY SERVICING,
- * REPAIR OR CORRECTION.  UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY,
- * WHETHER TORT (INCLUDING NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL THE
- * INITIAL DEVELOPER, THE UNIVERSITY OF BRITISH COLUMBIA, IMAGE POWER, INC.,
- * MICHAEL DAVID ADAMS, ANY OTHER CONTRIBUTOR, OR ANY DISTRIBUTOR OF THE
- * JASPER SOFTWARE, OR ANY SUPPLIER OF ANY OF SUCH PARTIES, BE LIABLE TO
- * THE USER OR ANY OTHER PERSON FOR ANY INDIRECT, SPECIAL, INCIDENTAL, OR
- * CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT LIMITATION,
- * DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR
- * MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF
- * SUCH PARTY HAD BEEN INFORMED, OR OUGHT TO HAVE KNOWN, OF THE POSSIBILITY
- * OF SUCH DAMAGES.  THE JASPER SOFTWARE AND UNDERLYING TECHNOLOGY ARE NOT
- * FAULT-TOLERANT AND ARE NOT DESIGNED, MANUFACTURED OR INTENDED FOR USE OR
- * RESALE AS ON-LINE CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING
- * FAIL-SAFE PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES,
- * AIRCRAFT NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT
- * LIFE SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
- * JASPER SOFTWARE OR UNDERLYING TECHNOLOGY OR PRODUCT COULD LEAD DIRECTLY
- * TO DEATH, PERSONAL INJURY, OR SEVERE PHYSICAL OR ENVIRONMENTAL DAMAGE
- * ("HIGH RISK ACTIVITIES").  LICENSOR SPECIFICALLY DISCLAIMS ANY EXPRESS
- * OR IMPLIED WARRANTY OF FITNESS FOR HIGH RISK ACTIVITIES.  USER WILL NOT
- * KNOWINGLY USE, DISTRIBUTE OR RESELL THE JASPER SOFTWARE OR UNDERLYING
- * TECHNOLOGY OR PRODUCTS FOR HIGH RISK ACTIVITIES AND WILL ENSURE THAT ITS
- * CUSTOMERS AND END-USERS OF ITS PRODUCTS ARE PROVIDED WITH A COPY OF THE
- * NOTICE SPECIFIED IN THIS SECTION.
+ * THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS
+ * LICENSE.  NO USE OF THE SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER
+ * THIS DISCLAIMER.  THE SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
+ * "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
+ * BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
+ * PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.  IN NO
+ * EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, OR ANY SPECIAL
+ * INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING
+ * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
+ * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
+ * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.  NO ASSURANCES ARE
+ * PROVIDED BY THE COPYRIGHT HOLDERS THAT THE SOFTWARE DOES NOT INFRINGE
+ * THE PATENT OR OTHER INTELLECTUAL PROPERTY RIGHTS OF ANY OTHER ENTITY.
+ * EACH COPYRIGHT HOLDER DISCLAIMS ANY LIABILITY TO THE USER FOR CLAIMS
+ * BROUGHT BY ANY OTHER ENTITY BASED ON INFRINGEMENT OF INTELLECTUAL
+ * PROPERTY RIGHTS OR OTHERWISE.  AS A CONDITION TO EXERCISING THE RIGHTS
+ * GRANTED HEREUNDER, EACH USER HEREBY ASSUMES SOLE RESPONSIBILITY TO SECURE
+ * ANY OTHER INTELLECTUAL PROPERTY RIGHTS NEEDED, IF ANY.  THE SOFTWARE
+ * IS NOT FAULT-TOLERANT AND IS NOT INTENDED FOR USE IN MISSION-CRITICAL
+ * SYSTEMS, SUCH AS THOSE USED IN THE OPERATION OF NUCLEAR FACILITIES,
+ * AIRCRAFT NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL
+ * SYSTEMS, DIRECT LIFE SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH
+ * THE FAILURE OF THE SOFTWARE OR SYSTEM COULD LEAD DIRECTLY TO DEATH,
+ * PERSONAL INJURY, OR SEVERE PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH
+ * RISK ACTIVITIES").  THE COPYRIGHT HOLDERS SPECIFICALLY DISCLAIM ANY
+ * EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR HIGH RISK ACTIVITIES.
  * 
  * __END_OF_JASPER_LICENSE__
  */
@@ -123,10 +74,15 @@
 * Includes.
 \******************************************************************************/
 
+/* The configuration header file should be included first. */
+#include <jasper/jas_config.h>
+
 #include <stdio.h>
+#if defined(JAS_HAVE_FCNTL_H)
 #include <fcntl.h>
+#endif
 #include <string.h>
-#if defined(HAVE_UNISTD_H)
+#if defined(JAS_HAVE_UNISTD_H)
 #include <unistd.h>
 #endif
 #include <jasper/jas_types.h>
@@ -159,7 +115,6 @@ extern "C" {
 /* The stream should be created/truncated. */
 #define JAS_STREAM_CREATE	0x0010
 
-#define JAS_STREAM_NOCLOSE	0x0020
 
 /*
  * Stream buffering flags.
@@ -254,24 +209,24 @@ typedef struct {
 	int flags_;
 
 	/* The start of the buffer area to use for reading/writing. */
-	uchar *bufbase_;
+	jas_uchar *bufbase_;
 
 	/* The start of the buffer area excluding the extra initial space for
 	  character putback. */
-	uchar *bufstart_;
+	jas_uchar *bufstart_;
 
 	/* The buffer size. */
 	int bufsize_;
 
 	/* The current position in the buffer. */
-	uchar *ptr_;
+	jas_uchar *ptr_;
 
 	/* The number of characters that must be read/written before
 	the buffer needs to be filled/flushed. */
 	int cnt_;
 
 	/* A trivial buffer to be used for unbuffered operation. */
-	uchar tinybuf_[JAS_STREAM_MAXPUTBACK + 1];
+	jas_uchar tinybuf_[JAS_STREAM_MAXPUTBACK + 1];
 
 	/* The operations for the underlying stream file object. */
 	jas_stream_ops_t *ops_;
@@ -291,8 +246,17 @@ typedef struct {
  * Regular file object.
  */
 
-/* Note: This is simply a file descriptor. */
-typedef int jas_stream_fileobj_t;
+/*
+ * File descriptor file object.
+ */
+typedef struct {
+	int fd;
+	int flags;
+	char pathname[L_tmpnam + 1];
+} jas_stream_fileobj_t;
+
+#define	JAS_STREAM_FILEOBJ_DELONCLOSE	0x01
+#define JAS_STREAM_FILEOBJ_NOCLOSE	0x02
 
 /*
  * Memory file object.
@@ -301,10 +265,10 @@ typedef int jas_stream_fileobj_t;
 typedef struct {
 
 	/* The data associated with this file. */
-	uchar *buf_;
+	jas_uchar *buf_;
 
 	/* The allocated size of the buffer for holding file data. */
-	int bufsize_;
+	size_t bufsize_;
 
 	/* The length of the file. */
 	int_fast32_t len_;
@@ -325,22 +289,26 @@ typedef struct {
 \******************************************************************************/
 
 /* Open a file as a stream. */
-jas_stream_t *jas_stream_fopen(const char *filename, const char *mode);
+JAS_DLLEXPORT jas_stream_t *jas_stream_fopen(const char *filename, const char *mode);
 
 /* Open a memory buffer as a stream. */
-jas_stream_t *jas_stream_memopen(char *buf, int bufsize);
+JAS_DLLEXPORT jas_stream_t *jas_stream_memopen(char *buf, int bufsize);
+
+/* Do not use this function.
+It will eventually replace jas_stream_memopen. */
+JAS_DLLEXPORT jas_stream_t *jas_stream_memopen2(char *buf, size_t bufsize);
 
 /* Open a file descriptor as a stream. */
-jas_stream_t *jas_stream_fdopen(int fd, const char *mode);
+JAS_DLLEXPORT jas_stream_t *jas_stream_fdopen(int fd, const char *mode);
 
 /* Open a stdio stream as a stream. */
-jas_stream_t *jas_stream_freopen(const char *path, const char *mode, FILE *fp);
+JAS_DLLEXPORT jas_stream_t *jas_stream_freopen(const char *path, const char *mode, FILE *fp);
 
 /* Open a temporary file as a stream. */
-jas_stream_t *jas_stream_tmpfile();
+JAS_DLLEXPORT jas_stream_t *jas_stream_tmpfile(void);
 
 /* Close a stream. */
-int jas_stream_close(jas_stream_t *stream);
+JAS_DLLEXPORT int jas_stream_close(jas_stream_t *stream);
 
 /******************************************************************************\
 * Macros/functions for getting/setting the stream state.
@@ -363,14 +331,14 @@ int jas_stream_close(jas_stream_t *stream);
 	(((const jas_stream_t *)(stream))->rwlimit_)
 
 /* Set the read/write limit for a stream. */
-int jas_stream_setrwlimit(jas_stream_t *stream, long rwlimit);
+JAS_DLLEXPORT int jas_stream_setrwlimit(jas_stream_t *stream, long rwlimit);
 
 /* Get the read/write count for a stream. */
 #define	jas_stream_getrwcount(stream) \
 	(((const jas_stream_t *)(stream))->rwcnt_)
 
 /* Set the read/write count for a stream. */
-long jas_stream_setrwcount(jas_stream_t *stream, long rwcnt);
+JAS_DLLEXPORT long jas_stream_setrwcount(jas_stream_t *stream, long rwcnt);
 
 /******************************************************************************\
 * Macros/functions for I/O.
@@ -391,19 +359,19 @@ long jas_stream_setrwcount(jas_stream_t *stream, long rwcnt);
 #endif
 
 /* Read characters from a stream into a buffer. */
-int jas_stream_read(jas_stream_t *stream, void *buf, int cnt);
+JAS_DLLEXPORT int jas_stream_read(jas_stream_t *stream, void *buf, int cnt);
 
 /* Write characters from a buffer to a stream. */
-int jas_stream_write(jas_stream_t *stream, const void *buf, int cnt);
+JAS_DLLEXPORT int jas_stream_write(jas_stream_t *stream, const void *buf, int cnt);
 
 /* Write formatted output to a stream. */
-int jas_stream_printf(jas_stream_t *stream, const char *fmt, ...);
+JAS_DLLEXPORT int jas_stream_printf(jas_stream_t *stream, const char *fmt, ...);
 
 /* Write a string to a stream. */
-int jas_stream_puts(jas_stream_t *stream, const char *s);
+JAS_DLLEXPORT int jas_stream_puts(jas_stream_t *stream, const char *s);
 
 /* Read a line of input from a stream. */
-char *jas_stream_gets(jas_stream_t *stream, char *buf, int bufsize);
+JAS_DLLEXPORT char *jas_stream_gets(jas_stream_t *stream, char *buf, int bufsize);
 
 /* Look at the next character to be read from a stream without actually
   removing it from the stream. */
@@ -412,47 +380,50 @@ char *jas_stream_gets(jas_stream_t *stream, char *buf, int bufsize);
 	  ((int)(*(stream)->ptr_)))
 
 /* Put a character back on a stream. */
-int jas_stream_ungetc(jas_stream_t *stream, int c);
+JAS_DLLEXPORT int jas_stream_ungetc(jas_stream_t *stream, int c);
 
 /******************************************************************************\
 * Macros/functions for getting/setting the stream position.
 \******************************************************************************/
 
 /* Is it possible to seek on this stream? */
-int jas_stream_isseekable(jas_stream_t *stream);
+JAS_DLLEXPORT int jas_stream_isseekable(jas_stream_t *stream);
 
 /* Set the current position within the stream. */
-long jas_stream_seek(jas_stream_t *stream, long offset, int origin);
+JAS_DLLEXPORT long jas_stream_seek(jas_stream_t *stream, long offset, int origin);
 
 /* Get the current position within the stream. */
-long jas_stream_tell(jas_stream_t *stream);
+JAS_DLLEXPORT long jas_stream_tell(jas_stream_t *stream);
 
 /* Seek to the beginning of a stream. */
-int jas_stream_rewind(jas_stream_t *stream);
+JAS_DLLEXPORT int jas_stream_rewind(jas_stream_t *stream);
 
 /******************************************************************************\
 * Macros/functions for flushing.
 \******************************************************************************/
 
 /* Flush any pending output to a stream. */
-int jas_stream_flush(jas_stream_t *stream);
+JAS_DLLEXPORT int jas_stream_flush(jas_stream_t *stream);
 
 /******************************************************************************\
 * Miscellaneous macros/functions.
 \******************************************************************************/
 
 /* Copy data from one stream to another. */
-int jas_stream_copy(jas_stream_t *dst, jas_stream_t *src, int n);
+JAS_DLLEXPORT int jas_stream_copy(jas_stream_t *dst, jas_stream_t *src, int n);
 
 /* Display stream contents (for debugging purposes). */
-int jas_stream_display(jas_stream_t *stream, FILE *fp, int n);
+JAS_DLLEXPORT int jas_stream_display(jas_stream_t *stream, FILE *fp, int n);
 
 /* Consume (i.e., discard) characters from stream. */
-int jas_stream_gobble(jas_stream_t *stream, int n);
+JAS_DLLEXPORT int jas_stream_gobble(jas_stream_t *stream, int n);
+
+/* Write a character multiple times to a stream. */
+JAS_DLLEXPORT int jas_stream_pad(jas_stream_t *stream, int n, int c);
 
 /* Get the size of the file associated with the specified stream.
   The specified stream must be seekable. */
-long jas_stream_length(jas_stream_t *stream);
+JAS_DLLEXPORT long jas_stream_length(jas_stream_t *stream);
 
 /******************************************************************************\
 * Internal functions.
@@ -481,15 +452,15 @@ directly, you will die a horrible, miserable, and painful death! */
 	  jas_stream_putc2(stream, c)) : EOF)
 #define jas_stream_putc2(stream, c) \
 	(((stream)->bufmode_ |= JAS_STREAM_WRBUF, --(stream)->cnt_ < 0) ? \
-	  jas_stream_flushbuf((stream), (uchar)(c)) : \
+	  jas_stream_flushbuf((stream), (jas_uchar)(c)) : \
 	  (++(stream)->rwcnt_, (int)(*(stream)->ptr_++ = (c))))
 
 /* These prototypes need to be here for the sake of the stream_getc and
 stream_putc macros. */
-int jas_stream_fillbuf(jas_stream_t *stream, int getflag);
-int jas_stream_flushbuf(jas_stream_t *stream, int c);
-int jas_stream_getc_func(jas_stream_t *stream);
-int jas_stream_putc_func(jas_stream_t *stream, int c);
+JAS_DLLEXPORT int jas_stream_fillbuf(jas_stream_t *stream, int getflag);
+JAS_DLLEXPORT int jas_stream_flushbuf(jas_stream_t *stream, int c);
+JAS_DLLEXPORT int jas_stream_getc_func(jas_stream_t *stream);
+JAS_DLLEXPORT int jas_stream_putc_func(jas_stream_t *stream, int c);
 
 #ifdef __cplusplus
 }
